/**
 * @mainpage VFIS Integral Equation Solver (Serial System Version).
 *
 * VFIS 
 *
 * Verified Fredholm Integral Equation Solver
 * for linear Fredholm Integral Equations of the second kind
 * (Serial System Version)
 *
 * Verifizierter Integralgleichungsloeser 
 * fuer lineare Fredholmsche Integralgleichungen zweiter Art
 * (Serielle Systemversion)
 *
 * Copyright (c) 2004-2007 Markus Grimmer
 * 
 * markus.grimmer@math.uni-wuppertal.de
 *
 * VFIS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * VFIS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with VFIS; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */


/**
 * @file System-FIGL-Loeser.cpp
 * VFIS Integral Equation Solver central module with all central methods/functions.
 * Class definitions see header file.
 */

#include <iostream>
#include <fstream>
#include <vector>
#include <ctime> 

#include "dim2taylor.hpp"  
#include "itaylor.hpp"     
#include "ilss.hpp"        

#include "interval.hpp"
#include "ivector.hpp"
#include "intvector.hpp"
#include "imatrix.hpp"
#include "real.hpp"

#include "dot.hpp"
#include "idot.hpp"

#include "vectoroperations.hpp"

#include "System-FIGL-Loeser.hpp"

using namespace std;
using namespace cxsc;
using namespace taylor;

// *****************************************************************************
// *****************************************************************************
// *****************************************************************************

// -----------------------------------------------------------------------------
// Auxiliary functions
// -----------------------------------------------------------------------------

/**
 * Start timer
 * @param[in] t1 Timer variable
 */
void start_clock(clock_t& t1)
{
   t1= clock();
   if (t1 == clock_t(-1))  //terminate if timer does not work properly
   {
      cerr << "Sorry, no clock\n";
      exit(1);
   }
   return;
}

/**
 * Print computation time
 * @param[in] t1   Timer variable
 * @param[in,out] outf Output file stream
 */
void print_time_used(clock_t t1, ostream& out)
{
   clock_t t2= clock();
   if (t2 == clock_t(-1))
   {
     cerr<< "Sorry, clock overflow\n";
     exit(2);
   }
   // Ausgabedatei!

   out << "Rechenzeit: " << 1000*double(t2-t1)/CLOCKS_PER_SEC
        << " msec" << endl;
   return;
} 

// -----------------------------------------------------------------------------

/**
 * Interval Midpoint Computation with interval result.
 * @param[in] I The interval
 * @return The midpoint
 */
interval Imid(interval& I)
{
  return (_interval(Inf(I))+_interval(Sup(I)))/_real(2.0);                        
}

// -----------------------------------------------------------------------------

/**
 * Epsilon inflation for itaylor arguments (elementwise).
 * @param[in]  it itaylor object
 * @param[in]  eps Epsilon
 * @return The inflated itaylor object
 */
itaylor blow(itaylor& it, real eps)
{
  return (_real(1.0)+eps)*it-eps*it;
}

/**
 * Epsilon inflation for vectors of itaylor arguments (elementwise).
 * @param[in]  vit vector of itaylor objects
 * @param[in]  eps Epsilon
 * @return Vector of inflated itaylor objects
 */
vector<itaylor> blow(vector<itaylor>& vit, real eps)
{
  vector<itaylor> ErgV0(vit.size(),itaylor(0));
  for (unsigned i=0; i<vit.size(); i++) //hier scheint "unsigned" n�ig zu sein.
    ErgV0[i]=blow(vit[i],eps);
  return ErgV0;
}

// -----------------------------------------------------------------------------

/**
 * Computes powers I^j, j=1..ord, for a given interval.
 * @param[in]  baseInt interval
 * @param[in]  ord     maximum exponent
 * @return ivector containing I^j, j=1..ord
 */
ivector Powers (interval baseInt, int ord)
{
  ivector resV(0,ord);
  resV[0]=_interval(1.0);
  
  for (int i=1; i<=ord; i++)
  {
    if (i%2==1) 
      resV[i]=baseInt*resV[i-1];
    else
      resV[i]=sqr(resV[i/2]);
  }
  
  return resV;
}

// -----------------------------------------------------------------------------

/**
 * Computes integrals of basic monomials.
 * @param[out]  Int_a0 Given domains T[i]:=[a_i,b_i], i=1..sysord 
                with midpoint t0_i, the integrals 
                of (a_i-t0_i)^k, k=0..intord, are computed and stored in Int_a[i] 
 * @param[out]  Int_b0 Given domains T[i]:=[a_i,b_i], i=1..sysord 
                with midpoint t0_i, the integrals 
                of (b_i-t0_i)^k, k=0..intord, are computed and stored in Int_b[i] 
 * @param[in]   sysord Order of integral equation system
 * @param[in]   intord Maximum order of integral computation
 * @param[in]   Tvec domains
 */
void Systembasis_Integration(imatrix& Int_a0, imatrix& Int_b0,
                             int sysord, int intord, ivector Tvec)
{
  interval Tmid;
  ivector a_Pot(0,intord+1), b_Pot(0,intord+1); 
  interval INull = interval(0.0,0.0);

  /*
  // Correct size should be set in constructor
  
  Resize(Int_a0,1,sysord,0,intord); 
  Resize(Int_b0,1,sysord,0,intord);
  */
  
  for (int i=1; i<=sysord; i++) 
  {
    Tmid = Imid(Tvec[i]);

    a_Pot=Powers(Inf(Tvec[i])-Tmid,intord+1);
    b_Pot=Powers(Sup(Tvec[i])-Tmid,intord+1);

    for (int j=0; j<=intord-1; j++)
    {
      Int_a0[i][j]=a_Pot[j+1]/_real(j+1);  
      Int_b0[i][j]=b_Pot[j+1]/_real(j+1);  
    }
    Int_a0[i][intord]=INull; 
    Int_b0[i][intord]=INull; 
    
  }
}

// *****************************************************************************
// *****************************************************************************
// *****************************************************************************

// -----------------------------------------------------------------------------
// class IntegralSystemOperatorRG:
//
// see Header
// -----------------------------------------------------------------------------

/**
 * Copy Constructor
 * @param[in]  IO An IntegralSystemOperatorRG
 */
IntegralSystemOperatorRG::IntegralSystemOperatorRG
                                            (const IntegralSystemOperatorRG& IO) 
{ 
  KSys0=IO.KSys0; 
  SSys=IO.SSys;
  SmidSys=IO.SmidSys;
}

// -----------------------------------------------------------------------------

/**
 * Constructor
 * @param[in]  vviv A vector<vector<ivector> >& for the kernel matrix
 * @param[in]  SSystem Domain of the (free) variable
 * @param[in]  SmidSystem Domain midpoints
 */
IntegralSystemOperatorRG::IntegralSystemOperatorRG
         (vector<vector<ivector> >& vviv, ivector& SSystem, ivector& SmidSystem) 
{ 
  KSys0=vviv; 
  SSys=SSystem;
  SmidSys=SmidSystem;
}

// -----------------------------------------------------------------------------

/**
 * Function call operator: Applies integral system operator to the function 
 * vector FktV0
 * @param[in]  FktV0  The function vector the integral operator is applied to
 * @param[in]  sysord Integral equation system order
 * @param[in]  ord Taylor order
 * @param[in]  BInt_a Integrals of powers of basic monomials
 * @param[in]  BInt_b Integrals of powers of basic monomials
 * @return     itaylor representation of the result vector
 */
vector<itaylor> IntegralSystemOperatorRG::operator()
                (vector<itaylor>& FktV0, int sysord, int ord, 
                 imatrix& BInt_a, imatrix& BInt_b)
{
  vector<itaylor> ErgV0(sysord,itaylor(ord)); 
  interval INull=_interval(0.0,0.0);
  interval Summe;
  ivector Tau1(0,ord+1), Tau2(0,ord+1);

  interval SmS0(0.0,0.0);    
  interval helpiv(0.0,0.0);

  for (int i=1; i<=sysord; i++)
  {
    Summe=INull;
    for (int j=1; j<=sysord; j++)     
    {
      SmS0 = SSys[i]-Imid(SSys[i]); 
      helpiv=KSys0[i-1][j-1][ord+1]*SmS0;
      for (int k=0; k<=ord; k++)
        Tau1[k]=FktV0[j-1][k]*helpiv;
      helpiv=KSys0[i-1][j-1][ord];
      for (int k=0; k<=ord; k++)
        Tau2[k]=FktV0[j-1][k]*helpiv;
      for (int k=0; k<=ord; k++)
        Summe+=Tau2[k]*BInt_b[j][k+1]-Tau2[k]*BInt_a[j][k+1]
              +Tau1[k]*BInt_b[j][k]-Tau1[k]*BInt_a[j][k];
    }//j

    ErgV0[i-1][ord]=Summe;
  }//i
  
  for (int i=1; i<=sysord; i++)
    for (int k=0; k<ord; k++) 
    {
      Summe=INull;
      for (int j=1; j<=sysord; j++)
      {
        helpiv=KSys0[i-1][j-1][k];  
        for (int l=0; l<=ord; l++)
          Tau1[l]=helpiv*FktV0[j-1][l];
        for (int l=0; l<=ord; l++)
          Summe+=Tau1[l]*BInt_b[j][ord+1-k+l]
                -Tau1[l]*BInt_a[j][ord+1-k+l];
      }//j
      ErgV0[i-1][k]=Summe;
    }//k

  return ErgV0;
}                 

// *****************************************************************************

/**
 * Part I: Iteration function: Carries out the fixed point iteration
 * 
 * y_0=g; y_{i+1}=IntOp(y_i)
 * 
 * for an integral system operator IntSysOp
 *
 * @param[in]  IntSysOp  The integral operator of the iteration
 * @param[in]  FktGV0    Right hand side function vector
 * @param[in]  sysord    Integral equation system order
 * @param[in]  ord       Taylor order
 * @param[in]  BInt_a    Integrals of powers of basic monomials
 * @param[in]  BInt_b    Integrals of powers of basic monomials
 * @param[out] err       Error code
 * @param[in,out] outf Output file stream
 * @return     itaylor representation of the result vector
 */
vector<itaylor> IterativesVerfahren (IntegralSystemOperatorRG& IntSysOp, 
                             vector<itaylor>& FktGV0, 
                             int sysord, int ord, 
                             imatrix& BInt_a, imatrix& BInt_b, int& err, ostream& outf)
{
  vector<itaylor> FAltV0(sysord,itaylor(0)), FNeuV0(sysord,itaylor(0));
  const real eps=1E-10;
  int itercount=0;

  FNeuV0=FktGV0;

  do
  {
    itercount++;
    FAltV0=blow(FNeuV0,eps);
    FNeuV0 = FktGV0 + IntSysOp(FAltV0, sysord, ord, BInt_a, BInt_b);
  } while (!(FNeuV0<=FAltV0)&&(itercount<=10));
  
  if(itercount>10)
  {
          cout << "Iteration: Keine Einschliessung!" << endl;
          err=1;
  }
  else
  {
          cout << "Iteration: Einschliessung nach " << itercount << " Schritten." << endl;
          err=0;    
  }
  
  return FNeuV0;
}

// *****************************************************************************
// *****************************************************************************
// *****************************************************************************

// -----------------------------------------------------------------------------
// class IntegralGleichungsSystem
//
// see Header
// -----------------------------------------------------------------------------

/**
 * Constructor for class IntegralGleichungsSystem
 * @param[in]  sysordng Order of integral equation system
 * @param[in]  ordng    Taylor order
 */
IntegralGleichungsSystem::IntegralGleichungsSystem(int sysordng, int ordng)
{
  sysord=sysordng;
  ord=ordng;
  const int guard=3;
  int intord=2*ord+guard;
  BasisInt_a = imatrix(1,sysord,0,intord);
  BasisInt_b = imatrix(1,sysord,0,intord);

  vector<KFkt> KFktvtmp = vector<KFkt>(sysord);
  KernFktSys0 = vector<vector<KFkt> >(sysord,KFktvtmp);
  RechtsFktSys0 = vector<ReFkt>(sysord);
  vector<dim2taylor> d2tvtmp = vector<dim2taylor>(sysord);
  KpointSys0 = vector<vector<dim2taylor> >(sysord,d2tvtmp);
  KintSys0 = vector<vector<dim2taylor> >(sysord,d2tvtmp);
  KernSys0 = vector<vector<dim2taylor> >(sysord,d2tvtmp);
  RSpointSys0 = vector<itaylor>(sysord,itaylor(0));
  RSintSys0 = vector<itaylor>(sysord,itaylor(0));
  RSSys0 = vector<itaylor>(sysord,itaylor(0));
  vector<ivector> ivvtmp = vector<ivector>(sysord);
  RestkernSys0 = vector<vector<ivector> >(sysord,ivvtmp);

  Sglob=interval(0.0);
  Tglob=interval(0.0);
  RS_Sglob=interval(0.0);
  SSys=ivector(sysord);
  TSys=ivector(sysord);
  RS_SSys=ivector(sysord);
  SmidSys=imatrix(sysord,sysord);
  TmidSys=imatrix(sysord,sysord);
  RS_SmidSys=ivector(sysord);
  
  ILGS_Lsg=ivector((ord+1)*sysord);
  Lsg0 = vector<itaylor>(sysord,itaylor(ord));
}

/**
 * Constructor for class IntegralGleichungsSystem
 * @param[in]  K        Kernel function (function pointer)
 * @param[in]  KS       Domain of the first variable of K 
 * @param[in]  KT       Domain of the second variable of K 
 * @param[in]  ReS      Right hand side function (function pointer)
 * @param[in]  GS       Domain of right hand side (expected to be =KS)
 * @param[in]  sysordng Order of integral equation system
 * @param[in]  ordng    Taylor order
 * @param[in,out] outf Output file stream
 */
IntegralGleichungsSystem::IntegralGleichungsSystem(KFkt K, interval KS, 
                          interval KT, ReFkt ReS, interval GS, 
                          int sysordng, int ordng, ostream& outf)
{
  real hS, hT;
  sysord=sysordng;
  ord=ordng;
 
  vector<KFkt> KFktvtmp = vector<KFkt>(sysord);
  KernFktSys0 = vector<vector<KFkt> >(sysord,KFktvtmp);
  RechtsFktSys0 = vector<ReFkt>(sysord);
  vector<dim2taylor> d2tvtmp = vector<dim2taylor>(sysord);
  KpointSys0 = vector<vector<dim2taylor> >(sysord,d2tvtmp);
  KintSys0 = vector<vector<dim2taylor> >(sysord,d2tvtmp);
  KernSys0 = vector<vector<dim2taylor> >(sysord,d2tvtmp);
  RSpointSys0 = vector<itaylor>(sysord,itaylor(0));
  RSintSys0 = vector<itaylor>(sysord,itaylor(0));
  RSSys0 = vector<itaylor>(sysord,itaylor(0));
  vector<ivector> ivvtmp = vector<ivector>(sysord);
  RestkernSys0 = vector<vector<ivector> >(sysord,ivvtmp);

  Sglob=KS;
  Tglob=KT;
  RS_Sglob=GS;

  SSys=ivector(sysord);
  TSys=ivector(sysord);
  SmidSys=ivector(sysord);
  TmidSys=ivector(sysord);
  RS_SmidSys=ivector(sysord);

  ILGS_Lsg=ivector((ord+1)*sysord);
  Lsg0 = vector<itaylor>(sysord,itaylor(ord));

  for (int i=0; i<sysord; i++)
  {
    for (int j=0; j<sysord; j++)
    {
      KernFktSys0[i][j]=K;
                          
    }
    RechtsFktSys0[i]=ReS; 
                          
  }  

  // Compute subdivided domains...

  hS=(Sup(Sglob)-Inf(Sglob))/real(sysord);
  hT=(Sup(Tglob)-Inf(Tglob))/real(sysord);

  // ...for S
  
  SetInf(SSys[1],Inf(Sglob));
  SetSup(SSys[1],Inf(Sglob)+hS);

  for (int i=1; i<=sysord-2; i++)
  {
    SetInf(SSys[i+1],Sup(SSys[i]));
    SetSup(SSys[i+1],Inf(SSys[i+1])+hS);
  }

  if (sysord>1)
  {
    SetInf(SSys[sysord],Sup(SSys[sysord-1]));
    SetSup(SSys[sysord],Sup(Sglob));
  }

  // ...for T 

  SetInf(TSys[1],Inf(Tglob));
  SetSup(TSys[1],Inf(Tglob)+hT);

  for (int i=1; i<=sysord-2; i++)
  {
    SetInf(TSys[i+1],Sup(TSys[i]));
    SetSup(TSys[i+1],Inf(TSys[i+1])+hT);
  }

  if (sysord>1)
  {
    SetInf(TSys[sysord],Sup(TSys[sysord-1]));
    SetSup(TSys[sysord],Sup(Tglob));

  }

  // ...for right hand side
  
  RS_SSys=SSys;

  // Midpoints

  for (int i=1; i<=sysord; i++)
  {
     SmidSys[i]=Imid(SSys[i]);
     TmidSys[i]=Imid(TSys[i]);
     RS_SmidSys[i]=Imid(RS_SSys[i]);
  }   

  // Integrals of basic monomials
  
  const int guard=3;
  int intord=2*ord+guard;
  BasisInt_a = imatrix(1,sysord,0,intord);
  BasisInt_b = imatrix(1,sysord,0,intord);

  Systembasis_Integration(BasisInt_a,BasisInt_b,sysord,intord,TSys);

  // Taylor expansion

  dim2taylor_vector tv;
  ivector KParam(2);

  for (int i=0; i<sysord; i++)
    for (int j=0; j<sysord; j++)
    {
      // Taylor expansion kernel:

      // (K.1) "Point" Expansion for interval midpoint

      KParam[1]=SmidSys[i+1]; 
      KParam[2]=TmidSys[j+1]; 
      tv=init_var(ord+1,KParam);
      KpointSys0[i][j]=KernFktSys0[i][j](tv);
   
      // (K.2) Interval expansion for domain interval (for remainder)
    
      KParam[1]=SSys[i+1]; 
      KParam[2]=TSys[j+1]; 
      tv=init_var(ord+1,KParam);
      KintSys0[i][j]=KernFktSys0[i][j](tv);
    
      // (K.3) Put them together
      
      RestkernSys0[i][j]=ivector(0,ord+1);
      KernSys0[i][j]=KpointSys0[i][j];
      
      for (int rgk=0; rgk<=ord+1; rgk++)
      {
        KernSys0[i][j][rgk][ord+1-rgk]=KintSys0[i][j][rgk][ord+1-rgk];
        RestkernSys0[i][j][rgk]=KintSys0[i][j][rgk][ord+1-rgk];
      }
 
    }//for j
  //(for i)
    
  // Taylor expansion right hand side
  
  for (int i=0; i<sysord; i++)
  {
  
    // (R.1/2) "point" and interval expansion
  
    itaylor x(ord,RS_SSys[i+1]);  
    itaylor xmid(ord,RS_SmidSys[i+1]);
    RSintSys0[i]=RechtsFktSys0[i](x);
    RSpointSys0[i]=RechtsFktSys0[i](xmid);
  
    // (R.3) Put them together
  
    RSSys0[i]=RSpointSys0[i];
    RSSys0[i][ord]=RSintSys0[i][ord];
  }
}                                     

// -----------------------------------------------------------------------------

/**
 * Main solution method containing method/subroutine calls for all parts of the 
 * integral equation solution method.
 * 
 * (All necessary data is stored in the class's attributes)
 *
 * @param[in,out] outf Output file stream
 */
void IntegralGleichungsSystem::Einschliessung_berechnen(ostream& outf)
{
  clock_t t_ges;      
  start_clock(t_ges); 
        
  vector<itaylor> EinsfuncV0(sysord,itaylor(0));

  vector<itaylor> vittmp(sysord,itaylor(0));
  vector<vector<itaylor> > vvittmp(sysord,vittmp);
  vector<vector<vector<itaylor> > > AlphaMatrix0(ord+2,vvittmp);
  vector<itaylor> F0(sysord,itaylor(0));
  vector<itaylor> AktAlpha(sysord,itaylor(0));

  interval INull=interval(0.0,0.0);
  interval IEins=interval(1.0,1.0);
  int err=0, errc=0;
  
  ivector Nullv(0,ord);
  Nullv=INull;
  itaylor ITNull; 
  ITNull=Nullv;
  for(int i=0; i<sysord; i++)
    EinsfuncV0[i]=ITNull;

  cout << "Starte Einschliessungsberechnung..." << endl;
  outf << "Starte Einschliessungsberechnung..." << endl;
  
  IntegralSystemOperatorRG ISORK(RestkernSys0,SSys,SmidSys);

  cout << "Starte iteratives Verfahren F." << endl;
  outf << "Starte iteratives Verfahren F." << endl;

  clock_t t_iter;      
  start_clock(t_iter); 

  F0=IterativesVerfahren(ISORK,RSSys0,sysord,ord,BasisInt_a,BasisInt_b,err,outf);

  cout << "Ausfuehrung iteratives Verfahren F beendet." << endl;
  outf << "Ausfuehrung iteratives Verfahren F beendet." << endl;

  int m=0;
  while ((m<=ord)&&(err==0))
  {
    if (m>0)
      EinsfuncV0[sysord-1][m-1]=INull;
                                      
    int i=1;
    while ((i<=sysord)&&(err==0))
    {

      if (i>1)
        EinsfuncV0[i-2][m]=INull;
                                
      EinsfuncV0[i-1][m]=IEins;

      cout << "Starte iteratives Verfahren AA["<<m<<"]["<<i-1<<"]..."<< endl;
      outf << "Starte iteratives Verfahren AA["<<m<<"]["<<i-1<<"]..."<< endl;

      AktAlpha=IterativesVerfahren(ISORK,EinsfuncV0,sysord,ord,
                                   BasisInt_a,BasisInt_b,errc,outf);

      err+=errc;
      if(err>0)      
      {
        cout << "Abbruch des iterativen Verfahrens bei: " << m << "," << i-1 << endl;
        outf << "Abbruch des iterativen Verfahrens bei: " << m << "," << i-1 << endl;
      }

      AlphaMatrix0[m][i-1]=AktAlpha;

      i++;
    }//while i
    m++;
  }//while m

  cout << "Zeit fuer Iterationen: " << endl;
  outf << "Zeit fuer Iterationen: " << endl;
  print_time_used(t_iter,cout);
  print_time_used(t_iter,outf);

  cout << "Fehler: " << err << endl;
  outf << "Fehler: " << err << endl;

  if (err==0)
  {
    SystemLoesungsansatz(AlphaMatrix0, F0, errc, outf);
    err+=errc;
  }
  
  if (err==0)
  {
    cout << "Berechne Loesung des Integralgleichungssystems..." << endl;
    outf << "Berechne Loesung des Integralgleichungssystems..." << endl;
    
    SystemLoesung_bilden(AlphaMatrix0, F0, errc, outf);
    err+=errc;
  }
  cout << "Fehler: " << err << endl;
  outf << "Fehler: " << err << endl;
  
  cout << "Gesamtzeit: " << endl;
  outf << "Gesamtzeit: " << endl;
  print_time_used(t_ges,cout);
  print_time_used(t_ges,outf);
  

} //void IntegralGleichungsSystem::Einschliessung_berechnen()

// -----------------------------------------------------------------------------

/**
 * Part II/III of the solution algorithm: Compute the elements of the linear system
 * and solve the system (the latter with the appropriate external function).
 * 
 * @param[in]  AM0  Iteration results
 * @param[in]  F0   Iteration result vector of single iteration
 * @param[out] err  Error code
 * @param[in,out] outf Output file stream
 */
void IntegralGleichungsSystem::SystemLoesungsansatz
     (vector<vector<vector<itaylor> > >& AM0, vector<itaylor>& F0, int err, ostream& outf)
{
  // AM: [0..ord+1,0..sysord-1,0..sysord-1]:[0..ord  ]!
  //     [Index id,    Index--,    Index--]:[Index id]
  //         vec        vec         vec      itaylor

  // F0: [0..sysord-1][0..ord]
  //     [Index--    ][Index id]
  //       vec         itaylor
  
  imatrix A((ord+1)*sysord,(ord+1)*sysord);
  imatrix Id((ord+1)*sysord,(ord+1)*sysord);
  ivector b((ord+1)*sysord);
  ivector Tau(0,ord+1);
  interval IHelp;
  interval Summe;
  interval INull=interval(0.0,0.0);
  vector<vector<dim2taylor> >& B0=KernSys0;

  for (int i=1; i<=(ord+1)*sysord; i++)
    for (int j=1; j<=(ord+1)*sysord; j++)
    {
      A[i][j]=interval(0.0,0.0);
      Id[i][j]=interval(0.0,0.0);
    }
  for (int i=1; i<=(ord+1)*sysord; i++)
    Id[i][i]=interval(1.0,1.0);


  cout << "Aufbauen eines Gleichungssystems der Dimension "
       << (ord+1)*sysord << " x " << (ord+1)*sysord << ":" << endl;
  outf << "Aufbauen eines Gleichungssystems der Dimension "
       << (ord+1)*sysord << " x " << (ord+1)*sysord << ":" << endl;

  // -----------------------
  // Compute System entries
  // -----------------------

  clock_t t_lgs_aufb;      
  start_clock(t_lgs_aufb); 

  // Block row
  
  for (int m=0; m<=ord; m++)
  {  
     // Block column
          
     for (int n=0; n<=ord; n++)  
     {
        for (int i=1; i<=sysord; i++)
          for (int j=1; j<=sysord; j++)
          {
            Summe=INull;
            
            for (int k=1; k<=sysord; k++ )
            {
              for (int mue=0; mue<=(ord-m); mue++)
              {
                IHelp=B0[i-1][k-1][m][mue];
                for (int nue=0; nue<=ord; nue++)
                {
                  Tau[nue]=AM0[n][k-1][j-1][nue]*IHelp;
                }//nue
                for (int nue=0; nue<=ord; nue++)
                {
                  Summe+=Tau[nue]*BasisInt_b[k][nue+mue]
                        -Tau[nue]*BasisInt_a[k][nue+mue];
                }//nue
              }//for mue
            }//for k

          A[m*sysord+i][n*sysord+j]=Summe;

          }//for (i,)j
     }//for n

    // Right hand side

    for (int i=1; i<=sysord; i++)
    {
      Summe=INull;
      for (int k=1; k<=sysord; k++)
      {
        for (int mue=0; mue<=ord-m; mue++)
        {
          IHelp=B0[i-1][k-1][m][mue];
          for (int nue=0; nue<=ord; nue++)
            Tau[nue]=F0[k-1][nue]*IHelp;
          for (int nue=0; nue<=ord; nue++)
            Summe+=Tau[nue]*BasisInt_b[k][nue+mue]
                  -Tau[nue]*BasisInt_a[k][nue+mue];
        }//for mue
      
      }//for k
      
      b[m*sysord+i]=Summe;
      
    }//for i
  
  }//for m

  // Subtraction of Identity

  A=Id-A; 

  cout << "Zeit fuer Aufbau des Gleichungssystems: " << endl;
  outf << "Zeit fuer Aufbau des Gleichungssystems: " << endl;
  print_time_used(t_lgs_aufb,cout);
  print_time_used(t_lgs_aufb,outf);

  // -----------------------
  // Solve system
  // -----------------------

  cout << "Fehler: " << err << endl;
  cout << "Gleichungssystem wird geloest..." << endl;
  outf << "Fehler: " << err << endl;
  outf << "Gleichungssystem wird geloest..." << endl;
  
  clock_t t_lgs_lsg;      
  start_clock(t_lgs_lsg); 

  LSS(A,b,ILGS_Lsg,err);
                               
  cout << "Zeit fuer Loesung des Gleichungssystems: " << endl;
  outf << "Zeit fuer Loesung des Gleichungssystems: " << endl;
  print_time_used(t_lgs_lsg,cout);
  print_time_used(t_lgs_lsg,outf);

  cout << "LGS-Loeser beendet." << endl;
  cout << "Fehler: " << err << endl;
  outf << "LGS-Loeser beendet." << endl;
  outf << "Fehler: " << err << endl;

}

// -----------------------------------------------------------------------------

/**
 * Part IV of the solution algorithm: Compute the solution from the linear
 * system solution.
 * 
 * @param[in]  AM0   Vector of iteration results
 * @param[in]  F0    Iteration result of single iteration
 * @param[out] err   Error code
 * @param[in,out] outf Output file stream
 */
void IntegralGleichungsSystem::SystemLoesung_bilden
     (vector<vector<vector<itaylor> > >& AM0, vector<itaylor>& F0, int err, ostream& outf)
{
  interval Summe;
  interval INull=interval(0.0,0.0);
  
  for (int i=1; i<=sysord; i++)
    for (int m=0; m<=ord; m++)  
      Lsg0[i-1][m]=INull; 

  if (err==0)
  {
    for (int i=1; i<=sysord; i++)
      for (int m=0; m<=ord; m++)
      {
        Summe=INull;
        for (int j=1; j<=sysord; j++)
          for (int n=0; n<=ord; n++)
            Summe+=AM0[m][j-1][i-1][n]*ILGS_Lsg[j+sysord*n];
        Lsg0[i-1][m]=F0[i-1][m]+Summe;
      }//for m
  }//if err

  real g=Genauigkeit(20);

  cout << "Loesung berechnen beendet. " << endl << endl;
  cout << "Endergebnis: " << endl
       << SetPrecision(25,15) << Lsg0 << endl;
  cout << "Genauigkeit/Stellen(bin., sampled): " << g << endl;
  
  outf << "Loesung berechnen beendet. " << endl << endl;
  outf << "Endergebnis: " << endl
       << SetPrecision(25,15) << Lsg0 << endl;
  outf << "Genauigkeit/Stellen(bin., sampled): " << g << endl; 

}

// -----------------------------------------------------------------------------

/**
 * Computes the result accuracy by sampling the function at a given 
 * number of evaluation points (evenly distributed)
 * 
 * @param[in]  Steps Number of sampling points
 * @return     Accuracy value
 */
real IntegralGleichungsSystem::Genauigkeit(int Steps)
{
  int i=1; 
  real D=-30.0;
  real h, s0, r1, r2;
  interval B;

  do
  {
    h=(Sup(SSys[i])-Inf(SSys[i]))/_real(Steps);
    s0=(Sup(SSys[i])+Inf(SSys[i]))/_real(2.0);

    int j=0;
    do
    {
      r1=Inf(SSys[i])+_real(j)*h;
      r2=r1-s0;
      B=Lsg0[i-1][ord];
      for (int k=ord; k>=1; k--) 
        B=B*_interval(r2)+Lsg0[i-1][k-1];
      if ((Inf(B)!=0.0) && !(!B))  // !B defined as "0.0 contained in B" in C-XSC
      {
        r1=abs(diam(B)/Inf(B));
        if (r1>D)
          D=r1;
      }
      j++;   
    } while (j<=Steps);
  i++;
  } while(i<=sysord);
  return -(ln(D)/ln(_real(2.0))); //Basis 2
}

// -----------------------------------------------------------------------------

/**
 * Export solution function in destination system's source code.
 * 
 * @param[in]  dest     Destination system (to date, only the string "Maple"
                        is supported)
 * @param[in]  fname    File name for code output 
 * @param[in]  varname  Variable prefix for function code
 */
void IntegralGleichungsSystem::Export(string dest, string fname, string varname)
{
  ofstream expfile(fname.c_str());
  
  if (dest=="Maple")
  {
     for (unsigned int i=0; i<Lsg0.size(); i++)
     {
       expfile << varname << "[" << i << "]" << " := x -> " << Lsg0[i][0];
       for (int j=1; j<=Lsg0[i].order(); j++)
       {
         expfile << " &+ ( " << Lsg0[i][j] 
                 << " &* ( ( x &- " << SmidSys[i+1] 
                 << " ) &intpower " << j << " ) ) ";
       }
       expfile << ";" << endl;
     }
  }
  else
  {
    cerr << "Export fehlgeschlagen: Zielsystem unbekannt."  << endl;
  }
}

/**
 * Export solution function and interval plot in destination system's 
 * source code.
 * 
 * @param[in]  dest      Destination system (to date, only the string "Maple"
                         is supported)
 * @param[in]  fname     File name for code output 
 * @param[in]  steps     Number of steps in function evaluation for plot
 * @param[in]  varname   Variable prefix for function code
 * @param[in]  plottitle Title for the plot
 */
void IntegralGleichungsSystem::Export_Plot(string dest, string fname, 
                                           int steps, string varname,  
                                           string plottitle)
{
  ofstream expfile(fname.c_str());

  interval x, val;
  real xstep;    
  
  if (dest=="Maple")
  {
    expfile << varname << "_Args := []:" << endl;
    expfile << varname << "_Vals := []:" << endl;
 
    for (unsigned int i=0; i<Lsg0.size(); i++)
    {
      xstep=(Sup(SSys[i+1])-Inf(SSys[i+1]))/real(steps);

      expfile << varname << "[" << i << "]" << " := x -> " << Lsg0[i][0];
      for (int j=1; j<=Lsg0[i].order(); j++)
      {
        expfile << " &+ ( " << Lsg0[i][j] 
                << " &* ( ( x &- " << SmidSys[i+1] 
                << " ) &intpower " << j << " ) ) ";
      }
      expfile << ";" << endl;

      for (int j=0; j<steps; j++)
      {
        if (j<(steps-1))
          x=interval(Inf(SSys[i+1])+real(j)*xstep,Inf(SSys[i+1])+real(j+1)*xstep);
        else
          x=interval(Inf(SSys[i+1])+real(j)*xstep,Sup(SSys[i+1]));

        val=Lsg0[i](x-interval(SmidSys[i+1]));

        expfile << varname << "_Args := [ op( " << varname << "_Args), "
                << x << " ]:" << endl;
        expfile << varname << "_Vals := [ op( " << varname << "_Vals), "
                << val << " ]:" << endl;
      }//j
    }//i
    
    expfile << varname << "_Plot := interval_list_plot( "
            << varname << "_Args, " << varname << "_Vals,"
            << " color=yellow, style=patch):" << endl;
    expfile << "display(" << varname 
             << "_Plot, axes=framed, title=\"" << plottitle << "\");" << endl;
  }
  else
  {
    cerr << "Export fehlgeschlagen: Zielsystem unbekannt."  << endl;
  }
}

// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------
// -----------------------------------------------------------------------------


/**
 * Output operator for class IntegralGleichungsSystem objects
 * 
 * @param[in,out]  os    Output stream
 * @param[in]      IGLS  IntegralGleichungsSystem object reference
 * @return         Output stream
 */
ostream& operator<< (ostream& os, IntegralGleichungsSystem& IGLS)
{
  os << "Integralgleichungssystem mit folgenden Funktionen:" << endl;
  os << "Kerne (Taylorentwicklungen):" << endl;
  
  for (int i=0; i<IGLS.sysord; i++)
    for (int j=0; j<IGLS.sysord; j++)
    {
      os << "KernSys[" << i << "][" << j << "]:" << endl;
      os << IGLS.KernSys0[i][j] << endl;
    }
  
  os << "Restkerne:" << endl;

  for (int i=0; i<IGLS.sysord; i++)
    for (int j=0; j<IGLS.sysord; j++)
    {
      os << "RestkernSys[" << i << "][" << j << "]:" << endl;
      os << IGLS.RestkernSys0[i][j] << endl;
    }

  os << "Rechte Seiten (Taylorentwicklungen):" << endl;

  for (int i=0; i<IGLS.sysord; i++)
  {
    os << "RSSys[" << i << "]:" << endl;
    os << IGLS.RSSys0[i] << endl;
  }
  return os;
}


// *****************************************************************************
// *****************************************************************************
// *****************************************************************************

