// ----------------------------------------------------------------------------
// This package is part of:
// ----------------------------------------------------------------------------
// VFIS 
// ----------------------------------------------------------------------------
// Verified Fredholm Integral Equation Solver
// for linear Fredholm Integral Equations of the second kind
// (Serial System Version)
// ----------------------------------------------------------------------------
// Verifizierter Integralgleichungsloeser 
// fuer lineare Fredholmsche Integralgleichungen zweiter Art
// (Serielle Systemversion)
// ----------------------------------------------------------------------------
// Copyright (c) 2004-2007 Markus Grimmer
//
// markus.grimmer@math.uni-wuppertal.de
// ----------------------------------------------------------------------------

// This file is part of the Verified Fredholm Integral Equation Solver VFIS.
// 
// VFIS is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// VFIS is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with VFIS; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------

/**
 * @file vectoroperations.cpp
 * Additional vector operators for STL vectors.
 */

#include <iostream>
#include <fstream>
#include <vector>
#include "vectoroperations.hpp"

#include "dim2taylor.hpp" 
#include "itaylor.hpp"    
#include "ivector.hpp"

using namespace std;
using namespace cxsc;
using namespace taylor;

template<class T> 
vector<T> operator+ (vector<T>& TV1, vector<T>& TV2)
{
  vector<T> ErgV;

  unsigned maxsize=max(TV1.size(),TV2.size());
  unsigned minsize=min(TV1.size(),TV2.size());

  for (unsigned i=0; i<minsize; i++)
    ErgV.push_back(TV1[i]+TV2[i]);
  if (TV1.size()>minsize)
    for (unsigned i=minsize; i<maxsize; i++)
      ErgV.push_back(TV1[i]);
  else
    for (unsigned i=minsize; i<maxsize; i++) 
      ErgV.push_back(TV2[i]);

  return ErgV;
}

template<class T> 
vector<T> operator+ (vector<T>& TV1, vector<T> TV2)
{
  vector<T> ErgV;

  unsigned maxsize=max(TV1.size(),TV2.size());
  unsigned minsize=min(TV1.size(),TV2.size());

  for (unsigned i=0; i<minsize; i++)
    ErgV.push_back(TV1[i]+TV2[i]);
  if (TV1.size()>minsize)
    for (unsigned i=minsize; i<maxsize; i++)
      ErgV.push_back(TV1[i]);
  else
    for (unsigned i=minsize; i<maxsize; i++)
      ErgV.push_back(TV2[i]);

  return ErgV;
}

template<class T> 
bool operator< (vector<T>& TV1, vector<T>& TV2)
{
  bool erg=true;
  if (TV1.size()!=TV2.size())
    erg=false;
  else
    for (unsigned i=0; i<TV1.size(); i++)
      if (!(TV1[i]<TV2[i]))
        erg=false;
  return erg;
}

template<class T> 
bool operator< (vector<T>& TV1, vector<T> TV2)
{
  bool erg=true;
  if (TV1.size()!=TV2.size())
    erg=false;
  else
    for (unsigned i=0; i<TV1.size(); i++)
      if (!(TV1[i]<TV2[i]))
        erg=false;
  return erg;
}

template<class T> 
bool operator<= (vector<T>& TV1, vector<T>& TV2)
{
  bool erg=true;
  if (TV1.size()!=TV2.size())
    erg=false;
  else
    for (unsigned i=0; i<TV1.size(); i++)
      if (!(TV1[i]<=TV2[i]))
        erg=false;
  return erg;
}

template<class T> 
bool operator<= (vector<T>& TV1, vector<T> TV2)
{
  bool erg=true;
  if (TV1.size()!=TV2.size())
    erg=false;
  else
    for (unsigned i=0; i<TV1.size(); i++)
      if (!(TV1[i]<=TV2[i]))
        erg=false;
  return erg;
}

template<class T> 
bool operator> (vector<T>& TV1, vector<T>& TV2)
{
  bool erg=true;
  if (TV1.size()!=TV2.size())
    erg=false;
  else
    for (unsigned i=0; i<TV1.size(); i++)
      if (!(TV1[i]>TV2[i]))
        erg=false;
  return erg;
}

template<class T> 
bool operator> (vector<T>& TV1, vector<T> TV2)
{
  bool erg=true;
  if (TV1.size()!=TV2.size())
    erg=false;
  else
    for (unsigned i=0; i<TV1.size(); i++)
      if (!(TV1[i]>TV2[i]))
        erg=false;
  return erg;
}

template<class T> 
bool operator>= (vector<T>& TV1, vector<T>& TV2)
{
  bool erg=true;
  if (TV1.size()!=TV2.size())
    erg=false;
  else
    for (unsigned i=0; i<TV1.size(); i++)
      if (!(TV1[i]>=TV2[i]))
        erg=false;
  return erg;
}

template<class T> 
bool operator>= (vector<T>& TV1, vector<T> TV2)
{
  bool erg=true;
  if (TV1.size()!=TV2.size())
    erg=false;
  else
    for (unsigned i=0; i<TV1.size(); i++)
      if (!(TV1[i]>=TV2[i]))
        erg=false;
  return erg;
}

template<class T> 
ostream& operator<< (ostream& os, vector<T>& TVec)
{
  for (unsigned i=0; i<TVec.size(); i++)
    os << "[" << i << "]: " << TVec[i] << endl;
  os << endl;
  return os;
}

template<class T> 
ofstream& operator<< (ofstream& os, vector<T>& TVec)
{
  for (unsigned i=0; i<TVec.size(); i++)
    os << "[" << i << "]: " << TVec[i] << endl;
  os << endl;
  return os;
}

// --- Explicit Instances: ------------------------------------------------

template vector<int> operator+ (vector<int>&, vector<int>&);
template vector<double> operator+ (vector<double>&, vector<double>&);
template vector<itaylor> operator+ (vector<itaylor>&, vector<itaylor>&);
template vector<ivector> operator+ (vector<ivector>&, vector<ivector>&);

template vector<int> operator+ (vector<int>&, vector<int>);
template vector<double> operator+ (vector<double>&, vector<double>);
template vector<itaylor> operator+ (vector<itaylor>&, vector<itaylor>);
template vector<ivector> operator+ (vector<ivector>&, vector<ivector>);

template bool operator< (vector<int>&, vector<int>&);
template bool operator< (vector<double>&, vector<double>&);
template bool operator< (vector<itaylor>&, vector<itaylor>&);
template bool operator< (vector<ivector>&, vector<ivector>&);
template bool operator< (vector<int>&, vector<int>);
template bool operator< (vector<double>&, vector<double>);
template bool operator< (vector<itaylor>&, vector<itaylor>);
template bool operator< (vector<ivector>&, vector<ivector>);

template bool operator<= (vector<int>&, vector<int>&);
template bool operator<= (vector<double>&, vector<double>&);
template bool operator<= (vector<itaylor>&, vector<itaylor>&);
template bool operator<= (vector<ivector>&, vector<ivector>&);
template bool operator<= (vector<int>&, vector<int>);
template bool operator<= (vector<double>&, vector<double>);
template bool operator<= (vector<itaylor>&, vector<itaylor>);
template bool operator<= (vector<ivector>&, vector<ivector>);

template bool operator> (vector<int>&, vector<int>&);
template bool operator> (vector<double>&, vector<double>&);
template bool operator> (vector<itaylor>&, vector<itaylor>&);
template bool operator> (vector<ivector>&, vector<ivector>&);
template bool operator> (vector<int>&, vector<int>);
template bool operator> (vector<double>&, vector<double>);
template bool operator> (vector<itaylor>&, vector<itaylor>);
template bool operator> (vector<ivector>&, vector<ivector>);

template bool operator>= (vector<int>&, vector<int>&);
template bool operator>= (vector<double>&, vector<double>&);
template bool operator>= (vector<itaylor>&, vector<itaylor>&);
template bool operator>= (vector<ivector>&, vector<ivector>&);
template bool operator>= (vector<int>&, vector<int>);
template bool operator>= (vector<double>&, vector<double>);
template bool operator>= (vector<itaylor>&, vector<itaylor>);
template bool operator>= (vector<ivector>&, vector<ivector>);

template ostream& operator<< (ostream&, vector<int>&);
template ostream& operator<< (ostream&, vector<double>&);
template ostream& operator<< (ostream&, vector<itaylor>&);
template ostream& operator<< (ostream&, vector<ivector>&);

template ofstream& operator<< (ofstream&, vector<int>&);
template ofstream& operator<< (ofstream&, vector<double>&);
template ofstream& operator<< (ofstream&, vector<itaylor>&);
template ofstream& operator<< (ofstream&, vector<ivector>&);



